/*
*************************************
file:  actions.c
*************************************
*/



#include "bmath.h"


#define dbg_flag 0



void print_wg_event(int *u);




/*
******************************************
Simulator for plant, controller
in closed-loop configuration.

u : event vector (size r)
x : state vector  (size k*n)

Variable order:
u_0, ... u_{n - 1}
x_{0, 0}, ... x_{0, n - 1}, ... 
x_{k - 1, 0}, ... x_{k - 1, n - 1}
******************************************
*/

void sim_closed_loop(
int k,   /*   number of state components   */
int r,   /* size event vector  */
int n,   /* size state vector (for each component)  */
bdd *P,   /* plant transition function  */
bdd *F   /* controller  */
)

{  /*  begin */

/* declarations  */


int i, j, h, not_end, jj;
int *x, *u, *z, *ux;
int max_in;

/*  program  */

#if (dbg_flag > 100)
printf("\ndet_controller: ckp 100 (begin)\n");
#endif

/* Define input, output vector    */ 

x = (int *) malloc(k*n * sizeof(int));
if (x == NULL)
{ 
    printf("\n\nError:  no space for x (sim_closed_loop)");  
    exit (1234);
}

u = (int *) malloc(r * sizeof(int));
if (u == NULL)
{ 
    printf("\n\nError:  no space for u (sim_closed_loop)");  
    exit (1234);
}

ux = (int *) malloc((r + k*n)* sizeof(int));
if (ux == NULL)
{ 
    printf("\n\nError:  no space for ux (sim_closed_loop)");  
    exit (1234);
}

z = (int *) malloc(k*n * sizeof(int));
if (z == NULL)
{ 
    printf("\n\nError:  no space for z (sim_closed_loop)");  
    exit (1234);
}





/* define initial state   */
for (i = 0; i < k*n; i++)
{
    x[i] = rand()%2;
}


/* begin simulation   */

not_end = 1;
while (not_end)
{   /*  while  */
	/*  print present state  */
	print_state(k, n, x);
	
	/* compute event via controller   */
	eval_bdd_vct(r, u, F, x, k*n);
	
	/* print event   */ 
	print_event(r, u);
	
	
	/* compute next state  */
	for (jj = 0; jj < r; jj++) { ux[jj] = u[jj]; }
	for (jj = 0; jj < k*n; jj++) { ux[r + jj] = x[jj]; }
	eval_bdd_vct(k*n, z, P, ux, r + k*n);

       /* update x and define not_end  */
       not_end = 0;
       	for (jj = 0; jj < k*n; jj++) 
       	{ 
       	    if (x[jj] != z[jj]) {not_end = 1;}
       	    x[jj] = z[jj];
       }
       
}    /*  while  */


free(x);
free(u);
free(ux);
free(z);


}  /*  end  */





void print_state(int k, int n, int *x)
{
/*  declarations  */
int i, j;

int *buf;

/*  program  */


buf = (int *) malloc(n* sizeof(int));

if (buf == NULL)
{ 
    printf("\n\nError:  no space for buf (print_state)");  
    exit (1234);
}


printf(
"\nstate vector (%d cmpts of size %d) = ",
k, n
);

for (j = 0; j < k; j++)
{


#if 0
/*  old version  */
	printf("\nstate.%d = ", j);
	for (i = 0; i < n; i++)
	{
		printf("%d ", x[j*n + i]);
	}
#endif


	for (i = 0; i < n; i++)
	{
		buf[i] = x[j*n + i];
	}

        printf("%u ", conv_bin_to_dec(n, buf));

}	


}   /*  end program  */



#if 0
/* general purpose version  */

void print_event(int r, int *u)
{
/*  declarations  */
int i;

/*  program  */

printf("\nevent vector (size %d) = ", r);

	for (i = 0; i < r; i++)
	{
		printf("%d ", u[i]);
	}

}   /*  end program  */

#endif





/*  wagon version   */


void print_event(int r, int *u)
{
/*  declarations  */
int i;
int buf[3];

/*  program  */

printf("\nevent vector (size %d) = ", r);

for (i = 0; i < r - 2; i = i + 3)
{
  buf[0] = u[i]; buf[1] = u[i + 1]; buf[2] = u[i + 2];
  print_wg_event(buf);
  printf("  ");
}


}   /*  end program  */



void print_wg_event(int *u)
{
/*  declarations  */
unsigned int buf;

/*  program  */

buf = conv_bin_to_dec(3, u);

	switch (buf)
	{
	     case 0: printf("nop");
		     break;

	     case 1: printf("forward");
		     break;

	     case 2: printf("back");
		     break;

	     case 3: printf("left");
		     break;

	     case 4: printf("right");
		     break;

	     default: printf("nonused event code: %u", buf);

	}

}   /*  end program  */
